from martian import baseclass
from grokcore.component import name
from grokcore.component import GlobalUtility

from zope.interface import implementer
from zope.component import getGlobalSiteManager
from zope.component.globalregistry import BaseGlobalComponents
from zope.component.globalregistry import GlobalAdapterRegistry

from bst.pygasus.core.interfaces import IApplicationContext
from bst.pygasus.core.interfaces import DEFAULT_EXTJS_APPLICATION


class LocalAdapterRegistry(GlobalAdapterRegistry):
    """ Local adapter/utilities registry
    """


class LocalComponentsRegistry(BaseGlobalComponents):
    """ The local component registry bound
        on application context.
    """
    
    def __init__(self, name):
        super(LocalComponentsRegistry, self).__init__(
            name=name,
            bases=(getGlobalSiteManager(),))

    def _init_registries(self):
        self.adapters = LocalAdapterRegistry(self, 'adapters')
        self.utilities = LocalAdapterRegistry(self, 'utilities')


@implementer(IApplicationContext)
class ApplicationContext(GlobalUtility):
    """ Represent a ExtJs Application. This is abstract
        class and need to subclass in your project.
    """
    baseclass()
    name(DEFAULT_EXTJS_APPLICATION)

    credentials_pluggins = ()
    authentication_pluggins = ()

    def __init__(self):
        super(ApplicationContext, self).__init__()
        self._sitemanager = LocalComponentsRegistry(self.application)

    def getSiteManager(self):
        return self._sitemanager

    def setSiteManager(self, sitemanager):
        self._sitemanager = sitemanager



from time import time
from urllib.parse import urljoin
from urllib.parse import urlparse

from grokcore import component
from zope.interface import implementer
from zope.component import queryUtility
from bst.pygasus.core.interfaces import IBaseUrl
from bst.pygasus.wsgi.interfaces import IRequest
from bst.pygasus.core.interfaces import IApplicationContext
from bst.pygasus.core.interfaces import DEFAULT_EXTJS_APPLICATION

X_FULL_PATH = 'X_FULL_PATH'


class ramcache(object):
    """ decorator to cache a function into ram
        arguments should be hashable
    """

    def __init__(self, expire):
        """ expire in miliseconds
        """
        self.expire = expire
        self.cache = dict()

    def __call__(self, func):
        def wrapper(*args, **kw):
            key = hash((args, tuple(kw),))
            cache = self.cache.setdefault(key, dict(last=0,
                                                    result=None))
            if cache['last'] + self.expire < time():
                cache['last'] = time()
                r = func(*args, **kw)
                cache['result'] = r
                return r
            else:
                return cache['result']
        return wrapper


@implementer(IBaseUrl)
class BaseUrl(component.Adapter):
    component.context(IRequest)

    def __init__(self, request):
        self.request = request

    def url(self, relativ_path=None):
        if X_FULL_PATH in self.request.headers:
            base_url = self.request.headers.get(X_FULL_PATH)
        else:
            base_url = self.request.application_url

        if not base_url.endswith('/'):
            base_url = '%s/' % base_url

        # if the publisher take a default utility. we
        # need rebuild the correct url.
        appname = urlparse(base_url).path.split('/')[-2:-1][0]
        if queryUtility(IApplicationContext, name=appname) is None:
            base_url = '%s%s/' % (base_url, DEFAULT_EXTJS_APPLICATION,)

        if relativ_path is None:
            return base_url

        if relativ_path.startswith('/'):
            relativ_path = relativ_path[1:]

        return base_url + relativ_path



from zope import schema
from zope.interface import Interface
from zope.interface import Attribute
from zope.component.interfaces import ISite

# main name for application context
DEFAULT_EXTJS_APPLICATION = 'mainapp'


class IApplicationContext(ISite):
    title = Attribute('title of application')
    resources = Attribute('fanstatic resources for all js and css')
    application = Attribute('classname of application entry point eg. "bielbienne.iptt.Application"')
    namespace = Attribute('Extjs name space eg: "bielbienne.iptt"')

    credentials_pluggins = Attribute('a list of utility names for credentials plugins')
    authentication_pluggins = Attribute('a list of utility names auth plugins')


class IBaseUrl(Interface):

    def __init__(self, request):
        """
            request: Client Request
        """

    def url(self, relativ_path=None):
        """ return base url or a url build
            with a given relative path
        """



import fanstatic

from grokcore import component
from zope.interface import implementer

from js.extjs import js
from js.extjs import basic

from bst.pygasus.core.interfaces import IApplicationContext
from bst.pygasus.resources.interfaces import IClassPathMapping
from bst.pygasus.i18n import i18njs


@implementer(IClassPathMapping)
class BaseClassPathMapping(component.Subscription):
    """ Base class for all extjs class path mapping registrations
    """
    component.baseclass()
    component.context(IApplicationContext)
    namespace = ''
    path = ''


class ExtClassPathMapping(BaseClassPathMapping):
    namespace = 'Ext'
    path = 'fanstatic/extjs/src/'


class BootstrapFanstaticResource(fanstatic.Resource):

    _depends = [i18njs, basic]

    def __init__(self):

        old_state = fanstatic.core._resource_file_existence_checking
        fanstatic.core._resource_file_existence_checking = False
        super(BootstrapFanstaticResource, self).__init__(fanstatic.Library('', ''), 'fake.js', depends=self._depends)
        fanstatic.core._resource_file_existence_checking = old_state

    def render(self, library_url):
        baseurl = library_url.rsplit('/', 2)[0]
        return self.renderer('%s/bootstrap' % baseurl)


class BootstrapFanstaticResourceSkinless(BootstrapFanstaticResource):
    _depends = [i18njs, js]

extjs_resources = BootstrapFanstaticResource()
extjs_resources_skinless = BootstrapFanstaticResourceSkinless()



""" This provide just all common function and directive for your project.
    import like this "bst.pygasus.core import ext" and you will be able
    to call all required stuff directly on "ext".
"""
import fanstatic

from grokcore.component import *
from zope.interface import implementer
from bst.pygasus.core.applicationcontext import ApplicationContext

from bst.pygasus.wsgi.interfaces import IRequest
from bst.pygasus.wsgi.events import IPreRequestProcessingEvent
from bst.pygasus.wsgi.events import IPostRequestProcessingEvent

from bst.pygasus.scaffolding.decorator import ScaffoldingDecorator as scaffolding

from js.extjs import basic

from bst.pygasus.core.resources import extjs_resources
from bst.pygasus.core.resources import extjs_resources_skinless
from bst.pygasus.core.resources import BaseClassPathMapping as ClassPathMapping

from bst.pygasus.datamanager.model import AbstractModelHandler
from bst.pygasus.datamanager.model import ExtBaseModel as Model
from bst.pygasus.datamanager.grokker import schema

from webob import exc



from setuptools import setup, find_packages
import os

version = '1.0.1'

long_description = (
    open('README.rst').read()
    + '\n' +
    open('CHANGES.rst').read()
    + '\n')

setup(name='bst.pygasus.core',
      version=version,
      description="Pygasus web framework working with ExtJs",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'License :: OSI Approved :: Zope Public License',
          'Operating System :: OS Independent',
          'Development Status :: 4 - Beta'
      ],

      keywords='pygasus web framework extjs',
      author='codeix',
      author_email='samuel.riolo@biel-bienne.ch',
      url='https://github.com/bielbienne/bst.pygasus.core',
      license='ZPL 2.1',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['bst', 'bst.pygasus'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'bst.pygasus.wsgi',
          'bst.pygasus.resources',
          'bst.pygasus.scaffolding',
          'bst.pygasus.datamanager',
          'bst.pygasus.security',
          'bst.pygasus.i18n',
          'js.extjs',
          'martian',
          'grokcore.component',
          'zope.interface',
          'Genshi'
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
